/*
 * Copyright (c) 1997 Secure Networks Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Secure Networks Inc.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>
#include <sys/malloc.h>
#include <sys/fcntl.h>
#include <termios.h>
#include "global.h"
#include "md5.h"

#define STATE_BUFFER_SIZE 16

/* ---------------------------------------------------------------------- */
unsigned char getrandombyte() {
  struct timeval tv;
  unsigned char somechar;
  int bytesread = 0;

  /* this could be more efficient with a select loop, but
     hopefully this code won't be run often.
     */
  while (bytesread < 1)
    bytesread = read (fileno (stdin), &somechar, 1);

  if (gettimeofday (&tv, NULL) < 0) {
    fprintf (stderr, "gettimeofday() failed: %s\n", strerror (errno));
    exit (-1);
  }

  return (tv.tv_usec % 256);
}

/* ---------------------------------------------------------------------- */
int getrandombuf (unsigned char *buf, int size) {
  int i;
  int attribs;
  struct termios origttystate, ttystate;
  char junk;

  printf ("Please enter random text on your keyboard until "
          "you see 'Enough!'\n");

  /* make it no delay */
  attribs = fcntl (fileno (stdin), F_GETFL,0);
  attribs |= O_NDELAY;
  fcntl (fileno(stdin), F_SETFL, attribs); 

  /* turn off editing */
  tcgetattr(0, &origttystate);
  memcpy (&ttystate, &origttystate, sizeof (struct termios));
  ttystate.c_lflag &= ~ICANON;
  ttystate.c_lflag &= ~ECHO;
  ttystate.c_cc[VMIN] = 1;
  ttystate.c_cc[VTIME] = 0;
  tcsetattr (0, TCSADRAIN, &ttystate);  

  for (i = 0 ; i < size ; i++) {
    buf[i] = getrandombyte();
    printf ("*");
    fflush (stdout);
  }
  printf ("\n\aEnough!\n");

  /* absorb extra junk */
  sleep (1); 
  while (read (fileno (stdin), &junk, 1) == 1) {
    /* do nothing */
  }  

  tcsetattr (0, TCSADRAIN, &origttystate);
  attribs &= ! O_NDELAY;
  fcntl (fileno (stdin), F_SETFL, attribs);

  return 0;
}


/* ---------------------------------------------------------------------- */
unsigned long getrand32() {
  unsigned char tempbuf [STATE_BUFFER_SIZE];
  unsigned long result;
  MD5_CTX context;
  static unsigned char *buf = NULL;
 
  if (buf == NULL) {
    buf = (unsigned char *)malloc (STATE_BUFFER_SIZE);
    if (buf == NULL) {
      fprintf (stderr, "malloc() failed\n");
      exit (-1);
    }
    getrandombuf (buf, STATE_BUFFER_SIZE);    
  }
  
  /* actually prepare the md5 hash of it */
  MD5Init (&context);
  MD5Update (&context, buf, STATE_BUFFER_SIZE);
  MD5Final (tempbuf, &context);
  /* update state info and return the result */  
  memcpy (buf, tempbuf, STATE_BUFFER_SIZE);
  memcpy (&result, buf, sizeof (int));
  return result;
}





